/*
Copyright 2017 - 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance with the License. A copy of the License is located at
    http://aws.amazon.com/apache2.0/
or in the "license" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and limitations under the License.
*/

const { DynamoDBClient } = require("@aws-sdk/client-dynamodb");
const {
  DeleteCommand,
  DynamoDBDocumentClient,
  GetCommand,
  PutCommand,
  QueryCommand,
  UpdateCommand,
  ScanCommand,
  BatchWriteCommand,
} = require("@aws-sdk/lib-dynamodb");
const awsServerlessExpressMiddleware = require("aws-serverless-express/middleware");
const bodyParser = require("body-parser");
const express = require("express");

const ddbClient = new DynamoDBClient({ region: process.env.TABLE_REGION });
const ddbDocClient = DynamoDBDocumentClient.from(ddbClient);

let tableName = "contacts";
if (process.env.ENV && process.env.ENV !== "NONE") {
  tableName = tableName + "-" + process.env.ENV;
}

const userIdPresent = true; // TODO: update in case is required to use that definition
const partitionKeyName = "contactsid";
const partitionKeyType = "S";
const sortKeyName = "";
const sortKeyType = "";
const hasSortKey = sortKeyName !== "";
const path = "/contacts";
const UNAUTH = "UNAUTH";
const hashKeyPath = "/:" + partitionKeyName;
const sortKeyPath = hasSortKey ? "/:" + sortKeyName : "";

// declare a new express app
const app = express();
app.use(bodyParser.json());
app.use(awsServerlessExpressMiddleware.eventContext());

// Enable CORS for all methods
app.use(function (req, res, next) {
  res.header("Access-Control-Allow-Origin", "*");
  res.header("Access-Control-Allow-Headers", "*");
  next();
});

// convert url string param to expected Type
const convertUrlType = (param, type) => {
  switch (type) {
    case "N":
      return Number.parseInt(param);
    default:
      return param;
  }
};

const getUserId = (req) => {
  const userId =
    req.apiGateway.event.requestContext.identity.cognitoAuthenticationProvider.split(
      ":CognitoSignIn:"
    )[1];

  return userId;
};

/************************************
 * HTTP Get method to list objects *
 ************************************/

app.get(path, async function (req, res) {
  var params = {
    TableName: tableName,
    Select: "ALL_ATTRIBUTES",
  };

  try {
    const data = await ddbDocClient.send(new ScanCommand(params));
    res.json(data.Items);
  } catch (err) {
    res.statusCode = 500;
    res.json({ error: "Could not load items: " + err.message });
  }
});

/*****************************************
 * HTTP Get method for get single object *
 *****************************************/

app.get(path + hashKeyPath, async function (req, res) {
  const params = {};

  params[partitionKeyName] = req.params[partitionKeyName];
  try {
    params[partitionKeyName] = convertUrlType(
      req.params[partitionKeyName],
      partitionKeyType
    );
  } catch (err) {
    res.statusCode = 500;
    res.json({ error: "Wrong column type " + err });
  }

  let getItemParams = {
    TableName: tableName,
    Key: params,
  };

  try {
    const data = await ddbDocClient.send(new GetCommand(getItemParams));
    if (data.Item) {
      res.json(data.Item);
    } else {
      res.json(data);
    }
  } catch (err) {
    res.statusCode = 500;
    res.json({ error: "Could not load items: " + err.message });
  }
});

/************************************
 * HTTP put method for insert object *
 *************************************/

app.put(path + "/edit", async function (req, res) {
  try {
    const userId = getUserId(req);
    if (userIdPresent) {
      req.body["userId"] = userId || UNAUTH;
    }
    const updateExpression = [];
    const expressionAttributeNames = {};
    const expressionAttributeValues = {};

    Object.entries(req.body).forEach(([key, value]) => {
      if (key !== partitionKeyName && key !== sortKeyName) {
        const placeholder = `#${key}`;
        updateExpression.push(`${placeholder} = :${key}`);
        expressionAttributeNames[placeholder] = key;
        expressionAttributeValues[`:${key}`] = value;
      }
    });
    // Update item in DynamoDB
    const updateItemParams = {
      TableName: tableName,
      Key: {
        [partitionKeyName]: req.body[partitionKeyName],
      },
      UpdateExpression: `SET ${updateExpression.join(", ")}`,
      ExpressionAttributeNames: expressionAttributeNames,
      ExpressionAttributeValues: expressionAttributeValues,
    };

    let data = await ddbDocClient.send(new UpdateCommand(updateItemParams));

    res.json({ success: "put call succeed!", url: req.url, data: data });
  } catch (err) {
    res.statusCode = 500;
    res.json({ error: err, url: req.url, body: req.body });
  }
});

/************************************
 * HTTP post method for insert object *
 *************************************/

app.post(path, async function (req, res) {
  const userId = getUserId(req);
  if (userIdPresent) {
    req.body["userId"] = userId || UNAUTH;
  }

  let putItemParams = {
    TableName: tableName,
    Item: req.body,
  };
  try {
    let data = await ddbDocClient.send(new PutCommand(putItemParams));
    res.json({ success: "post call succeed!", url: req.url, data: data });
  } catch (err) {
    res.statusCode = 500;
    res.json({ error: err, url: req.url, body: req.body });
  }
});

/**************************************
 * HTTP remove method to delete object *
 ***************************************/

app.delete(path + "/delete" + hashKeyPath, async function (req, res) {
  const params = {};

  params[partitionKeyName] = req.params[partitionKeyName];
  try {
    params[partitionKeyName] = convertUrlType(
      req.params[partitionKeyName],
      partitionKeyType
    );
  } catch (err) {
    res.statusCode = 500;
    res.json({ error: "Wrong column type " + err });
  }

  let removeItemParams = {
    TableName: tableName,
    Key: params,
  };

  try {
    let data = await ddbDocClient.send(new DeleteCommand(removeItemParams));
    res.json({ url: req.url, data: data });
  } catch (err) {
    res.statusCode = 500;
    res.json({ error: err, url: req.url });
  }
});

app.patch(path + "/updateFavorite" + hashKeyPath, async function (req, res) {
  try {
    const userId = getUserId(req);
    if (userIdPresent) {
      req.body["userId"] = userId || UNAUTH;
    }
    const { isFavorite, contactsid } = req.body;

    // Update item in DynamoDB
    const updateItemParams = {
      TableName: tableName,
      Key: {
        [partitionKeyName]: req.body[partitionKeyName],
      },
      ExpressionAttributeNames: {
        "#isFavorite": "isFavorite",
      },
      ExpressionAttributeValues: {
        ":isFavorite": isFavorite,
      },
      UpdateExpression: "SET #isFavorite = :isFavorite",
      ReturnValues: "ALL_NEW",
    };

    const data = await ddbDocClient.send(new UpdateCommand(updateItemParams));

    // Extract only the isFavorite value
    const updatedIsFavorite = data.Attributes
      ? data.Attributes.isFavorite
      : null;

    res.json({
      success: "update call succeed!",
      url: req.url,
      isFavorite: updatedIsFavorite,
    });
  } catch (err) {
    res.statusCode = 500;
    res.json({ error: err, url: req.url, body: req.body });
  }
});

app.delete(path + "/batch-remove", async function (req, res) {
  const userId = getUserId(req);
  if (userIdPresent) {
    req.body["userId"] = userId || UNAUTH;
  }

  const contactIdsToDelete = req.body.contactsids;

  if (!contactIdsToDelete || !Array.isArray(contactIdsToDelete)) {
    res.statusCode = 400;
    res.json({
      error: "Invalid or missing contactIds array in the request body",
    });
    return;
  }

  const deleteRequests = contactIdsToDelete.map((contactId) => {
    return {
      DeleteRequest: {
        Key: {
          [partitionKeyName]: contactId,
        },
      },
    };
  });

  const params = {
    RequestItems: {
      [tableName]: deleteRequests,
    },
  };

  try {
    let data = await ddbDocClient.send(new BatchWriteCommand(params));
    res.json({ url: req.url, data: data });
  } catch (err) {
    res.statusCode = 500;
    res.json({ error: err, url: req.url });
  }
});

app.listen(3000, function () {
  console.log("App started");
});

// Export the app object. When executing the application local this does nothing. However,
// to port it to AWS Lambda we will create a wrapper around that will load the app from
// this file
module.exports = app;
