import { useState } from "react";
import Card from "@mui/material/Card";
import { DataGrid } from "@mui/x-data-grid";
import { useLocation, useNavigate } from "react-router-dom";

import Page from "components/Page";
import DataGridToolbar from "components/DataGridToolbar";
import useContactsColumns from "./useContactsColumns";

import useConfirmDialog from "hooks/useConfirmDialog";
import { CONTACTS_ROUTES } from "routes/paths";
import {
  useDeleteSingleContactMutation,
  useGetAllContactsQuery,
  useUpdateIsFavoriteStatusContactMutation,
  useBatchDeleteContactMutation,
} from "api/contacts";

import { ContactEntry } from "types";
import firstCharToUpperCase from "utils/firstCharToUpperCase";

export default function AllContacts() {
  const { pathname } = useLocation();
  const [pageSize, setPageSize] = useState<number>(5);
  const [selectedIds, setSelectedIds] = useState<string[]>([]);

  const { data: allContactsData, isLoading: allContactsLoading } =
    useGetAllContactsQuery(undefined);

  const [batchDeleteContact] = useBatchDeleteContactMutation();

  const [deleteOneContact] = useDeleteSingleContactMutation();
  const [updateIsFavoriteStatusContact] =
    useUpdateIsFavoriteStatusContactMutation();

  const [getConfirmation, Confirmation] = useConfirmDialog();
  const navigate = useNavigate();

  function filterContacts(data: ContactEntry[]) {
    const parts = pathname.split("/").filter(Boolean);
    const urlTarget = parts[parts.length - 1];

    if (urlTarget === "list") {
      return data;
    } else if (urlTarget === "favorites") {
      const favoriteContacts = data.filter(
        (contact) => contact.isFavorite === true
      );
      return favoriteContacts;
    } else {
      const filteredContactsByLabel = data.filter((contact) => {
        return contact.labels.some(
          (label) => label === firstCharToUpperCase(urlTarget)
        );
      });
      return filteredContactsByLabel;
    }
  }

  const handleBatchDelete = async () => {
    const isConfirmed = await getConfirmation({
      title: "Delete contacts",
      contentText: "Are you sure you want to delete these contacts?",
      confirmLabel: `Delete`,
    });
    console.log(selectedIds);
    if (isConfirmed) batchDeleteContact(selectedIds);
  };

  const handleFavorite = (contact: ContactEntry) => {
    const updatedFavoriteStatus = {
      ...contact,
      contactsid: contact.contactsid,
      isFavorite: !contact.isFavorite,
    };

    updateIsFavoriteStatusContact(updatedFavoriteStatus);
  };

  const handleEdit = (id: string) => {
    navigate(`${CONTACTS_ROUTES.edit}/${id}`);
  };
  const handleDelete = async (id: string) => {
    const isConfirmed = await getConfirmation({
      title: "Delete contact",
      contentText: "Are you sure you want to delete this contact?",
      confirmLabel: `Delete`,
    });
    isConfirmed && deleteOneContact(id);
  };

  const columns = useContactsColumns({
    handleFavorite,
    handleDelete,
    handleEdit,
  });

  let filteredContacts: ContactEntry[] | undefined = [];

  if (allContactsData) {
    filteredContacts = filterContacts(allContactsData);
  }

  return (
    <Page title="All contacts">
      <Card sx={{ m: 5 }}>
        <DataGrid
          pageSize={pageSize}
          onPageSizeChange={(newPageSize: number) => setPageSize(newPageSize)}
          rowsPerPageOptions={[5, 10, 20, 50, 100]}
          initialState={{
            sorting: { sortModel: [{ field: "name", sort: "asc" }] },
          }}
          rows={filteredContacts || []}
          columns={columns}
          loading={allContactsLoading}
          getRowId={(row) => row.contactsid}
          components={{ Toolbar: DataGridToolbar }}
          componentsProps={{
            toolbar: {
              numSelected: selectedIds.length,
              onDelete: handleBatchDelete,
            },
          }}
          onSelectionModelChange={(ids) => setSelectedIds(ids as string[])}
          disableExtendRowFullWidth={true}
        />
        <Confirmation />
      </Card>
    </Page>
  );
}
